//
// © 2013 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//
/* globals ROT, Faction, console */
// NOTE: Game is NOT Global here - we get called BEFORE Game is fully initialised
var GUILDS = {
    "Dairiers": 1,
    "Poulterers": 1,
    "Fishmongers": 2,
    "Fruiterers": 1,
    "Bakers & Pastriers": 1,
    "Brewer & Hostellers": 3,
    "Meatmongers": 1,
    "Cornmongers": 1,
    "Chandlers": 1,
    "Vintners": 1,
    "Mercers": 6,
    "Grossers": 5,
    "Drapers & Tailors": 2,
    "Clothworkers & Weavers": 2,
    "Fletchers": 2,
    "Bowyers": 1,
    "Coopers": 2,
    "Broiderers": 1,
    "Skinners & Leathersellers": 1,
    "Saddlers & Harnessers": 1,
    "Dyers": 1,
    "Watermen": 1,
    "Plasterers": 1,
    "Painters": 1,
    "Thatchers": 1,
    "Carpenters": 2,
    "Glaziers": 1,
    "Goldsmiths": 2,
    "Shipwrights": 1,
    "Masons": 1,
    "Apothecaries": 1
};

function Elector(index,name,gender,chance,address,guild) {
    guild = guild || 0.1;
    this._index = index;
    this._name = name;
    this._gender = gender;
    this._address = address;
    address.addInhabitant(this);
    this._chance = chance;
    this._alderman = (index === 1); // Initially
    this._councillor = (index > 1 && index < 14); // Initially 

    if (guild.constructor === String) {
        this._guild = guild;
        this._eligible = true;
    } else if (ROT.RNG.getUniform() < guild) {
        this._guild = ROT.RNG.getWeightedValue(GUILDS);
        this._eligible = true;
    } else {
        this._guild = null;
        this._eligible = false;
    }
    
    

    this._traits = [];

    this._traits.addone = function(choices) {
        var choice = choices.random();
        if (choice)  { this.push(choice); }
    };

    this._traits.addone(["curious",undefined,undefined,"cautious"]);
    this._traits.addone(["organised",undefined,undefined,"relaxed"]);
    this._traits.addone(["outgoing",undefined,undefined,"reserved"]);
    this._traits.addone(["nervous",undefined,undefined,"confident"]);
    this._traits.addone(["generous",undefined,undefined,"miserly"]);

    this._worries = {crime:0, money:0, plague:0, invasion:0, fire:0, famine:0};
    while(ROT.RNG.getPercentage() <= 50) {
        var worry = ["crime","money","plague","invasion","fire","famine"].random();
        this._worries[worry] += 1;
    }
    
    if (this._chance < 0.9) {
        this._keepVoteSecretChance = ROT.RNG.getUniform();
    } else {
        this._keepVoteSecretChance = 0;
    }
    this._preferences = []; // No one initially
    this._faction = undefined;
}

Elector.prototype = {
    getName: function() {
        return this._name;
    },
    getGender: function() {
        return this._gender;
    },
    getForename: function() {
        return this._name.split(' ')[0];
    },
    getSurname: function() {
        return this._name.split(' ')[1];
    },
    getAddress: function() {
        return this._address;
    },
    getGuild: function() {
        return this._guild;
    },
    addWorry: function(w) {
        if (this._worries[w]) {
            this._worries[w] += 1;
        }
    },
    getIndex: function() {
        return this._index;
    },
    getFirstPreference: function() {
        return this._firstPreference;
    },
    getOriginalPreference: function() {
        return this._originalPreference;
    },
    getVoted: function() {
        return this._voted;
    },
    decideIfVoted: function(flag) {
        this._voted = (Math.random() < this._chance);
    },
    decideIfSecretlyVoted: function() {
        this._lastVoteSecret = (Math.random() < this._keepVoteSecretChance);
    },
    getSecretlyVoted: function(flag) {
        return this._lastVoteSecret;
    },
    setAlderman: function(flag) {
        this._alderman = flag;
    },
    setCouncillor: function(flag) {
        this._councillor = flag;
    },
    setInitialPreference: function(index,electorate) {
        while(index === null || !electorate[index]._guild) {
            if (ROT.RNG.getPercentage() <= 30) {
                index = 1; // The alderman
            } else if (ROT.RNG.getPercentage() <= 20) {
                index = 2; // The known name of a councillor
            } else if (ROT.RNG.getPercentage() <= 20) {
                index = 3; // The known name of a councillor
            } else if (ROT.RNG.getPercentage() <= 20) {
                index = 4; // The known name of a councillor
            } else if (ROT.RNG.getPercentage() <= 50) {
                index = [5,6,7,8,9,10,11,12,13].random(); // indexes 5 through 13 are hereby also councillors
            } else {
                index = ROT.RNG.getUniformInt(0,electorate.length-1); // Could be anyone (including the player! or a non-guildmember)
            }
        }
        this._firstPreference = index;
        this._originalPreference = index;
        this._preferences = [];
        this._preferences[0] = 0.001; // Start low (but, generously, non-zero) for the player
        this._preferences[1] = ROT.RNG.getUniform();
        for(var i=2; i<5; i++) {
            this._preferences[i]=ROT.RNG.getUniform()*0.8; // Known councillors get 80% of the score (to allow alderman to probably win
        }
        for(i=5; i<14; i++) {
            this._preferences[i]=ROT.RNG.getUniform()*0.8; // Known councillors get 80% of the score (to allow alderman to probably win
        }
        this._preferences[index] = 1; // But their first (initial) choice gets the maximum score - at least initially
    },
    setEligible: function(flag) {
        this._eligible = flag;
    },
    isEligible: function() {
        return this._eligible;
    },
    getEligibleMultiplier: function() {
        if (this._eligible) {
            return 1;
        } else {
            return 0;
        }
    },
    computeFirstPreference: function(electorate) {
        var bi = 0, b = this._preferences[0];
        var name = this._name;
        this._preferences.forEach(function(v, i) {
            console.log(name+"'s score for "+electorate[i].getName()+" is now "+v*electorate[i].getEligibleMultiplier());
            if (v*electorate[i].getEligibleMultiplier() > b) {
                bi = i;
                b = v*electorate[i].getEligibleMultiplier();
            }
        });
        this._firstPreference = bi;
    },
    saveFirstPreference: function() {
        this._votedFor = this._firstPreference;
    },
    getVotedFor: function() {
        return this._votedFor;
    },
    improvePreference: function(electorate,amount,index) {
        if (amount <= 0) {
            return false; // Ignore negative improvements
        } else if (amount > 1) {
            amount = 1;
        }
        index = index || 0;
        var oldp;
        if (!this._preferences[index]) {
            oldp = 0;
            this._preferences[index] = amount;
        } else {
            oldp = this._preferences[index];
            this._preferences[index] = 1 - (1-this._preferences[index])*(1-amount);
        }

        // Everyone else goes down a bit as well
        this._preferences.forEach(function(v, i, a){
            if (index !== i) {
                a[i] = v * (1-amount/4);
            }
        });
        var oldFirst = this._firstPreference;
        this.computeFirstPreference(electorate);
        if (oldFirst !== this._firstPreference) {
            console.log(this.getName()+" has changed their preference from "+electorate[oldFirst].getName()+" to "+electorate[this._firstPreference].getName());
        }
        return oldFirst !== this._firstPreference;
    },
    diminishPreference: function(electorate,amount,index) {
        if (amount <= 0) {
            return false; // Ignore negative diminish
        } else if (amount > 1) {
            amount = 1;
        }
        index = index || 0;
        if (this._preferences[index]) {
            this._preferences[index] *= (1-amount);
            var oldFirst = this._firstPreference;
            this.computeFirstPreference(electorate);
            if (oldFirst !== this._firstPreference) {
                console.log(this.getName()+" has changed their preference from "+electorate[oldFirst].getName()+" to "+electorate[this._firstPreference].getName());
            }
            return oldFirst !== this._firstPreference;
        } else {
            return false; // Can't go lower than 0 (or non-existent) and it certainly doesn't change the first preference
        }
    },
    setVoteChance: function(chance) {
        this._chance = chance;
    },
    setName: function(name) {
        this._name = name;
    },
    halveVoteChance: function() {
        this._chance = this._chance/2;
    },
    getFocusScore: function(focus,factors) {
        var fakeline = {};
        fakeline[focus] = 10;
        fakeline.getRelevance = function(aspect) {
            if (this[aspect] !== undefined) {
                return this[aspect];
            } else {
                return 5;
            }
        };
        return this.getScoreAdjustments(factors,fakeline);
    },
    hir: function() {
        if (this._gender === "M") {
            return "his";
        } else {
            return "her";
        }
    },
    Hir: function() {
        if (this._gender === "M") {
            return "His";
        } else {
            return "Her";
        }
    },
    she: function() {
        if (this._gender === "M") {
            return "he";
        } else {
            return "she";
        }
    },
    She: function() {
        if (this._gender === "M") {
            return "He";
        } else {
            return "She";
        }
    },
    description: function(electorate,factors,isa) {
        var d;
        isa = isa || ' is';

        if (this._guild) {
            d = this._name+isa+" a member of the worshipful company of "+this._guild+".";
        } else {
            d = this._name+isa+" not a member of any guild.";
        }

        if (this._name === "YOU") {
            d += "You voted for yourself in the last election.";
        } else {
            var she = this.She();
            if (this._alderman) {
                d += " "+she+" is the Alderman of Oldgate.";
            } else if (this._councillor) {
                d += " "+she+" is one of the councillors for Oldgate.";
            } else if (this._lastVoteSecret) {
                d += " "+she+" does not wish to reveal whether or how they voted in the last election.";
            } else if (this._voted) {
                d += " "+she+" voted for "+electorate[this._votedFor].getName()+" in the last election.";
            } else {
                d += " "+she+" did not vote in the last election, but would have voted for "+electorate[this._firstPreference].getName()+".";
            }

            var listit = function(array,sprefix,pprefix) {
                pprefix = pprefix || sprefix;
                if (array.length === 0) {
                    return "";
                }
                var result = (array.length === 1 ? sprefix : pprefix);
                if (array.length > 1) {
                    if (array.length > 2) {
                        for(var i=array.length-1; i>1; i--) {
                            result += array[i]+", ";
                        }
                    }
                    result += array[1]+" and ";
                }
                result += array[0]+".";
                return result;
            };
            d += listit(this._traits," "+she+" is ");

            var cs = [];
            var ws = [];
            var ps = [];
            for(var w in this._worries) {
                if (this._worries.hasOwnProperty(w)) {
                    var beads = this._worries[w]*factors[w];
                    if (beads >= 4) {
                        ps.push(w);
                    } else if (beads >= 2) {
                        ws.push(w);
                    } else if (beads > 0.5) {
                        cs.push(w);
                    }
                }
            }
            d += listit(cs," "+she+" has a concern about "," "+she+" has concerns about ");
            d += listit(ws," "+she+" is worried about ");
            d += listit(ps," "+she+" is paranoid about ");
        }
            
        return d;
    },
    _addToScore: function(why,delta) {
        if (delta !== 0) {
            this._scoreDetails += ", "+why+" adds "+delta;
        }
        return delta;
    },
    _plusScore: function(why, score, factor) {
        return this._addToScore(why,(score-5)*factor);
    },
    _minusScore: function(why, score, factor) {
        return this._addToScore(why,(5-score)*factor);
    },
    getScoreAdjustments: function(factors,line) {
        var adj = 0;
        var self = this;
        var tf = {
            curious:  {plus: ["fun"], minus: []},
            cautious: {plus: ["money","formal"], minus:["fun"]},
            organised:{plus: ["work","formal", "money"], minus: []},
            relaxed:  {plus: ["fun"], minus: ["formal"]},
            outgoing: {plus: ["fun"], minus: ["formal"]},
            reserved: {plus: ["work","formal"], minus: ["fun"]},
            nervous:  {plus: ["crime","formal","plague","invasion","fire","famine"], minus: []},
            confident:{plus: [], minus: ["formal"]},
            generous: {plus: ["fun"], minus: ["formal","money"]},
            miserly:  {plus: ["formal","money"], minus: ["fun"]}
        };
        
        this._traits.forEach(function(t){
            for(var i=0; i<tf[t].plus.length; i++) {
                adj += self._plusScore(t+"[+"+tf[t].plus[i]+"]",line.getRelevance(tf[t].plus[i]),2);
            }
            for(i=0; i<tf[t].minus.length; i++) {
                adj += self._minusScore(t+"[-"+tf[t].minus[i]+"]",line.getRelevance(tf[t].minus[i]),2);
            }
       });
        
        for(var w in this._worries) {
            if (this._worries.hasOwnProperty(w)) {
                adj += this._plusScore("worry["+w+"]",line.getRelevance(w),this._worries[w]*factors[w]*2);
            }
        }
        
        return adj;
    },
    getScore: function(factors,line) {
        var score = line.getReason().getScore(line)*5;
        this._scoreDetails = "baseScore="+score;
        
        // Now modify score based on our address
        var delta = line.getReason().scoreDeltaForAddress(this.getAddress());
        if (delta !== 0) {
            score += this._addToScore("address",line.getReason().scoreDeltaForAddress(this.getAddress()));
        }
        
        score += this.getScoreAdjustments(factors,line);
        score += this._addToScore("random",ROT.RNG.getUniformInt(-25,25));
        
        if (score < 0)   { score = 0; }
        if (score > 100) { score = 100; }
        
        return score; // 0 to 100
    },
    getSimilarity: function(other) {
        if (this === other) { 
            return 1;
        }
        var sim = 0.5;
        
        for(var t in this._traits) {
            if (this._traits.hasOwnProperty(t)) {
                for(var ot in other._traits) {
                    if (other._traits.hasOwnProperty(ot)) {
                        if (t === ot) {
                            sim += 0.05;
                        }
                    }
                }
            }
        }

        for(var w in this._worries) {
            if (this._worries.hasOwnProperty(w)) {
                for(var ow in other._worries) {
                    if (other._worries.hasOwnProperty(ow)) {
                        if (w === ow && this._worries[w] > 0 && other._worries[ow] > 0) {
                            sim += this._worries[w]*other._worries[ow] / (this._worries[w]*this._worries[w] + other._worries[ow]*other._worries[ow]) / 10;
                        }
                    }
                }
            }
        }

        return sim;
    },
    reply: function(factors,score,line) {
        console.log("(reply: "+this._scoreDetails+". total="+score+"%)");
        return line.getReason().getReply(this,score); 
    },
    replyWork: function(score) {
        if (score < 25) {
            return "Never! I'm happy with %a's paintwork.";
        } else if (score < 60) {
            return "Well maybe later, perhaps next year.";
        } else if (score < 90) {
            return "Yes, the paintwork does need doing, let's get that arranged.";
        } else {
            return "Absolutely, let's get this paintwork done!";
        }
    },
    replyFun: function(score) {
        if (score < 25) {
            return "Go Away!";
        } else if (score < 40) {
            return "Hmm, I'm not sure I like you.";
        } else if (score < 60) {
            return "Let's have another pint.";
        } else if (score < 78) {
            return "Hey, you're good.";
        } else {
            return "When's the next party!";
        }
    },
    replyTell: function(score) {
        if (score < 10) {
            return "Tempting - but not for you!";
        } else if (score < 60) {
            return "What's the point?";
        } else if (score < 90) {
            return "Sure that's a good idea.";
        } else {
            return "Absolutely.";
        }
    },
    replyCampaign: function(score) {
        if (score < 20) {
            return "Never!";
        } else if (score < 50) {
            return "Hmm, I'll consider it.";
        } else if (score < 75) {
            return "Yeah, you sound good, I'll vote for you.";
        } else {
            return "Absolutely.";
        }
    },
    replySmear: function(score) {
        if (score < 20) {
            return "I'm still going to vote for %c.";
        } else if (score < 50) {
            return "Hmm, I'll consider it.";
        } else if (score < 75) {
            return "Yeah, you're right. %d isn't for me.";
        } else {
            return "Absolutely.";
        }
    },
    replyWardmoot: function(score) {
        if (score < 20) {
            return "You're going to ignore me - I can tell.";
        } else if (score < 50) {
            return "Hmm, I'll consider it.";
        } else if (score < 75) {
            return "Yeah, please do discuss that at the wardmoot.";
        } else {
            return "Absolutely.";
        }
    },
    replyWatch: function(score) {
        if (score < 20) {
            return "You'll never catch them.";
        } else if (score < 50) {
            return "I don't think the watch will get my stuff back.";
        } else if (score < 75) {
            return "Yea! You're going to sort it out for me!";
        } else {
            return "So no more crime.....";
        }
    },
    replyWitanmoot: function(score) {
        if (score < 20) {
            return "You're going to ignore me - I can tell.";
        } else if (score < 50) {
            return "Hmm, I'll consider it.";
        } else if (score < 75) {
            return "Yeah, please do discuss that at the Witanmoot.";
        } else {
            return "Absolutely.";
        }
    },
    replyPromote: function(score) {
        if (score < 40) {
            return "Why?";
        } else if (score <60) {
            return "I don't see how that affects me though!";
        } else {
            return "Absolutely!";
        }
    },
    getFaction: function() {
        return this._faction;
    },
    setFaction: function(f) {
        this._faction = f || Faction.random();
    }
};

Elector.scoreWork = function(line) {
    return line.getRelevance("work");
};
Elector.scoreFun = function(line) {
    return line.getRelevance("fun");
};
Elector.scoreTell = function(line) {
    return 5;
};
Elector.scoreCampaign = function(line) {
    return 5;
};
Elector.scoreSmear = function(line) {
    return 5;
};
Elector.scoreWardmoot = function(line) {
    return 5;
};
Elector.scoreWatch = function(line) {
    return line.getRelevance("crime");
};
Elector.scoreWitanmoot = function(line) {
    return 5;
};
Elector.scorePromote = function(line) {
    return 5;
};
Elector.replyWork = function(elector,score) {
    return elector.replyWork(score);
};
Elector.replyFun = function(elector,score) {
    return elector.replyFun(score);
};
Elector.replyTell = function(elector,score) {
    return elector.replyTell(score);
};
Elector.replyCampaign = function(elector,score) {
    return elector.replyCampaign(score);
};
Elector.replySmear = function(elector,score) {
    return elector.replySmear(score);
};
Elector.replyWardmoot = function(elector,score) {
    return elector.replyWardmoot(score);
};
Elector.replyWatch = function(elector,score) {
    return elector.replyWatch(score);
};
Elector.replyWitanmoot = function(elector,score) {
    return elector.replyWitanmoot(score);
};
Elector.replyPromote = function(elector,score) {
    return elector.replyPromote(score);
};
