//
// © 2016 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//
/* globals Reason, module, exports */

function Firstline(reasonCode,topic,view,attitude,line) {
    this._reason = Reason.forCode(reasonCode);
    this._topic = topic;
    this._view = view;
    this._attitude = attitude;
    this._line = line;
}


Firstline._reasons = {};

Firstline.getReasons = function() {
    var rs = [];
    for(var p in Firstline._reasons) {
        if (Firstline._reasons.hasOwnProperty(p)) {
            rs.push(p);
        }
    }
    return rs;
};

Firstline.getTopics = function(reason) {
    if (reason instanceof Reason) {
        reason = reason.getCode();
    }
    var ts = [];
    for(var p in Firstline._reasons[reason]) {
        if (Firstline._reasons[reason].hasOwnProperty(p)) {
            ts.push(p);
        }
    }
    return ts;
};

Firstline.getViews = function(reason,topic) {
    if (reason instanceof Reason) {
        reason = reason.getCode();
    }
    var vs = [];
    if (Firstline._reasons[reason].hasOwnProperty(topic)) {
        for(var p in Firstline._reasons[reason][topic]) {
            if (Firstline._reasons[reason][topic].hasOwnProperty(p)) {
                vs.push(p);
            }
        }
    }
    return vs;
};

Firstline.getAttitudes = function(reason,topic,view) {
    if (reason instanceof Reason) {
        reason = reason.getCode();
    }
    var as = [];
    if (Firstline._reasons[reason].hasOwnProperty(topic)) {
        if (Firstline._reasons[reason][topic].hasOwnProperty(view)) {
            for(var p in Firstline._reasons[reason][topic][view]) {
                if (Firstline._reasons[reason][topic][view].hasOwnProperty(p)) {
                    as.push(p);
                }
            }
        }
    }
    return as;
};

Firstline.getLine = function(reason,topic,view,attitude) {
    if (reason instanceof Reason) {
        reason = reason.getCode();
    }
    reason = reason || "undefined";
    return Firstline._reasons[reason][topic][view][attitude];
};

Firstline.addLine = function(reason,topic,view,attitude,line) {
    if (reason instanceof Reason) {
        reason = reason.getCode();
    }

    if (reason && Array.isArray(reason)) {
        reason.forEach(function(r){
            Firstline.addLine(r,topic,view,attitude,line);
        });
    } else if (reason && !Reason.isValidCode(reason)) {
        throw new Error("Unrecognized Reason '"+reason+"'");
    } else if (reason) {
        if (!Firstline._reasons[reason]) {
            Firstline._reasons[reason] = {};
        }
        if (!Firstline._reasons[reason][topic]) {
            Firstline._reasons[reason][topic] = {};
        }
        if (!Firstline._reasons[reason][topic][view]) {
            Firstline._reasons[reason][topic][view] = {};
        }
        Firstline._reasons[reason][topic][view][attitude] = new Firstline(reason,topic,view,attitude,line);
    } else {
        for(var p in Firstline._reasons) {
            if (Firstline._reasons.hasOwnProperty(p)) {
                Firstline.addLine(p,topic,view,attitude,line);
            }
        }
    }
};

Firstline.prototype = {
    getText: function() {
        return this._line.text;
    },
    getRelevance: function(aspect) {
        if (this._line[aspect] !== undefined) {
            return this._line[aspect];
        } else {
            return 5;
        }
    },
    getReason: function() {
        return this._reason;
    }
};

Firstline.addLine("work","Work","Guild","Efficient", {text:"It's quick and easy to get a house painted with the Guild of Painters", fun:4, work:8, money:4});
Firstline.addLine("work","Work","Guild","Modest", {text:"Well the other guildmembers are busy, so I guess I could try this for you", work:6, formal:3});
Firstline.addLine("work","Work","Guild","Friendly", {text:"It won't be that expensive with the Guild of Painters guarantee", work:6, money:4, formal:4});
Firstline.addLine("work","Work","Guild","Formal", {text:"The Worshipful Company of Painters guarantees my work.", money:8, fun:3, work:8, formal:9});
Firstline.addLine("work","Work","Guild","Sympathic", {text:"Wow, who did this work? It needs fixing right away.", work:3, formal:3, money:6});
Firstline.addLine("work","Work","Guild","Informal", {text:"Let me and my guildies fix this up pronto - we'll have it look great again", work:6, formal:2});
Firstline.addLine("work","Work","Guild","Aloof", {text:"Let me pass you onto one of my colleagues", work:3, fun:6, money:3, formal:4});
Firstline.addLine("work","Work","Guild","Proud", {text:"I'm of the best painters in the Guild, so this'll be no problem", work:6, money:6, fun:4, formal:4});
Firstline.addLine("work","Work","Guild","Persuasive", {text:"I'd like to use your house as an example of my work, so do we have a deal?", work:6, fun:4, money:4, formal:4});

Firstline.addLine("work","Work","Painting","Efficient", {text:"I can get %a painted up in the next week or so.", fun:0, work:10, formal:6});
Firstline.addLine("work","Work","Painting","Modest", {text:"Well it'll be a difficult job, but I'll get it done.", fun:0, work:10});
Firstline.addLine("work","Work","Painting","Friendly", {text:"That's what I like to see, a nice easy job.", fun:0, work:10, formal:3});
Firstline.addLine("work","Work","Painting","Formal", {text:"I'll start first thing next week.", money:8, fun:0, work:10, formal:7});
Firstline.addLine("work","Work","Painting","Sympathic", {text:"Wow, this paint is in a bad way - I'll get this job done soon", crime:6, money:3, fun:0, work:10});
Firstline.addLine("work","Work","Painting","Informal", {text:"Yeah, this'll be no problem - I'll do it soon.", fun:1, work:9, formal:3});
Firstline.addLine("work","Work","Painting","Aloof", {text:"I'm sure I can fit %a into my schedule.", money:8, fun:0, work:10, formal:8});
Firstline.addLine("work","Work","Painting","Proud", {text:"Sure, it'll be no problem.", fun:0, work:8, formal:4});
Firstline.addLine("work","Work","Painting","Persuasive", {text:"Hmm, the paint here could really do with some work.", money:8, fun:0, work:10, formal:5});

Firstline.addLine("fun","Pub","Quiet Drink","Friendly", {text:"Let's go for a quiet drink", crime:4, fun:7, work:3, formal:3});
Firstline.addLine("fun","Pub","Quiet Drink","Conspiratorial", {text:"Let's go for a quiet drink and make plans", crime:3, fun:4, work:6, formal:4});
Firstline.addLine("fun","Pub","Quiet Drink","Business-like",{text:"Let's discuss your painting job over a quiet pint", work:6, fun:4, formal:4});

Firstline.addLine("fun","Pub","Pub Crawl", "Friendly", {text: "Fancy a pub crawl?", crime:2, money:4, fun:8, work:2, formal:1});
Firstline.addLine("fun","Pub","Pub Crawl", "Party!", {text:"Let's have a party!", crime:2, money:2, fun:10, work:2, formal:0});

Firstline.addLine("fun","Theatre","Latest Play","Friendly", {text: "What do you think of the latest play by Peter Sanderson?", money:4, fun:8, formal:3, plague:0, invasion:0, famine:0});

Firstline.addLine("fun","Dice","Gamble","Friendly", {text: "How are you at knucklebones?", crime:3, money:6, fun:8, work:3, formal:3, plague:0, invasion:0, famine:0});

Firstline.addLine(["tell","campaign"],"Persuade","Vote","Friendly", {text: "Can you help me get elected this year?", fun:3, work:3});
Firstline.addLine(["tell","campaign"],"Persuade","Vote","Formal", {text: "Can I count on your vote this year?", fun:4, work:4, formal:8});
Firstline.addLine(["tell","campaign"],"Persuade","Vote","Proud", {text: "I'll be proud to be your councillor in the coming year.", fun:4, work:4, format:6});
Firstline.addLine("tell","Persuade","Vote","Persuasive", {text: "Can I persuade you to go out and vote this year?", fun:4, work:4, formal:7});
Firstline.addLine("campaign","Persuade","Vote","Persuasive", {text: "Can I persuade you to vote for me this year?", fun:4, work:4, formal:7});

Firstline.addLine("smear","Persuade","Don't Vote","Friendly", {text: "You know that %d hasn't done anything to help around here.", crime:4, money:4, plague:4, invasion:4, fire:4, famine:4});
Firstline.addLine("smear","Persuade","Don't Vote","Persuasive", {text: "Please don't vote for %d.", fun:4, work:4, formal:7});

Firstline.addLine("smear","City Politics","Walls","Waste of Money", {text: "%d wants to build up the walls, what a waste of money", money:8, invasion:0 });
Firstline.addLine("smear","City Politics","Walls","Needed", {text: "We could be invaded at any time and %d refuses to support improving the city walls!", money:2, invasion:10 });
Firstline.addLine("smear","City Politics","Sweet Water","Waste of Money", {text: "%d wants to build an aqueduct through the Torian Temple - what a stupid idea.", money:8, plague:0 });
Firstline.addLine("smear","City Politics","Sweet Water","Needed", {text: "We're threatened by disease and %d doesn't support the sweet water project - very dangerous.", money:0, plague:10 });
Firstline.addLine("smear","City Politics","Civic Ball","Waste of Money", {text: "Your taxes are going up and all %d wants to do is attend the next civic ball.", money:0, fun:0, work:8 });
Firstline.addLine("smear","City Politics","Civic Ball","Needed", {text: "We need to increase trade and %d doesn't support the necessary wining and dining.", money:8, fun:10, work:3 });

Firstline.addLine("watch","Recruit","Join the watch","Persuasive", {text: "Join the Watch, your ward needs you!", crime:10, money:2, fun:0, work:4, formal:10});
Firstline.addLine("watch","Recruit","Join the watch","Friendly", {text: "We are looking for brave citizens of Oldgate to join the ward watch and help protect our beloved city! If you're looking for a way to make a difference and serve your community, then we invite you to join us!", crime:9, money:2, fun:1, work:4, formal:8, gpt:true});

Firstline.addLine("sherrif","Promote","Sherrif","Persuasive", {text: "Haven't we got a good alderman? %d should be one of the Sherrifs!", formal:3});
Firstline.addLine("sherrif","Promote","Lord","Persuasive", {text: "Haven't we got a good alderman? %d should be Lord of the City!", formal:3});
Firstline.addLine("sherrif","Gossip","Lord","whisper", {text:"Have you heard what the Lord of the City is up to?", money:6, fun:6, work:4, formal:3});
Firstline.addLine("sherrif","Gossip","Sherrifs","whisper", {text:"Have you heard what the Sherrifs are up to?", crime:8, money:4, fun:3, work:4, formal:3});

Firstline.addLine("wardmoot","Burglary","Their home","Friendly", {text: "I heard you've been robbed. Shall I complain to the wardmoot?", crime:8, money:6, fun:2, work:4, formal:3});

Firstline.addLine("witanmoot","Burglary","Their home","Friendly", {text: "I heard you've been robbed. Shall I complain to the witanmoot?", crime:10, money:4, fun:1, work:3, formal:4});

Firstline.addLine(["tell","campaign","smear"],"Gossip","Alderman","whisper",{text:"Have you heard about %d?", money:6, fun:4, work:4, formal:4});
Firstline.addLine(["tell","campaign","smear"],"Gossip","Wardmoot","whisper",{text:"Have you heard about what the wardmoot is up to?", crime:3, money:3, fun:6, work:6, formal:3});

Firstline.addLine(["work","tell","campaign","smear"],"Gossip","Guild of Painters", "whisper", {text: "Have you heard about what my guild is up tp?", crime:4, money:7, work:3, formal:3});

Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Lots of","Efficient", {text:"I'll get the ward watch to quickly send those miscreants off to the lockups.", crime:10, money:3, fun:1, formal:7});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Lots of","Modest", {text:"I'll try my best to help the ward watch catch the miscreants.", crime:8, money:4, fun:0, work:2});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Lots of","Friendly", {text:"Gosh, What a mess! Let me help clean this up!", crime:6, fun:4, work:3, formal:2});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Lots of","Formal", {text:"We must expand the watch patrols and destroy these criminals.", crime:10, money:3, fun:0, work:2, formal:10});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Lots of","Sympathic", {text:"It's a bit of a mess around here, but we can sort it out!", crime:7, money:4, fun:6, work:4, formal:2});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Lots of","Informal", {text:"It's those toshers again, let's sort 'em out.", crime:7, money:7, fun:1, work:4, formal:2});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Lots of","Aloof", {text:"Is that a Hue and Cry? Let's get 'em.", crime:9, fun:1, work:1});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Lots of","Proud", {text:"We can't allow the criminals to get established in Oldgate. Help me to stop them.", crime:7, money:3, fun:3, work:3});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Lots of","Persuasive", {text:"There does seem to be a lot of crime around. We need to sort it out now!", crime:7, money:4, fun:3, work:3});

Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Ward Watch","Efficient", {text:"The ward watch has captured lots of criminals.", crime:4, money:3, fun:3, work:4, formal:3});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Ward Watch","Modest", {text:"Yeah, I help out with the ward watches patrol", crime:6, money:4, fun:3, work:4, formal:4});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Ward Watch","Friendly", {text:"Have you asked the ward watch to help?", crime:6, money:3, fun:3, work:2, formal:4});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Ward Watch","Formal", {text:"Can you help the ward watch with their inquiries.", crime:7, fun:2, work:4, formal:9});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Ward Watch","Sympathic", {text:"You mean the ward watch haven't helped? Let me sort that out.", crime:8, money:4, fun:3, work:3, formal:3});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Ward Watch","Informal", {text:"Yeah, the ward watch are investigating, but I think we'll need to help them out some more.", crime:7, money:4, fun:3, work:4, formal:2});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Ward Watch","Aloof", {text:"I wasn't on that patrol, but I'm sure that the ward watch did the right thing.", crime:7, money:3, fun:3, formal:8});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Ward Watch","Proud", {text:"The ward watch are doing an excellent job.", crime:7, fun:3, formal:7});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","Ward Watch","Persuasive", {text:"Could you join the ward watch and help us sort out the crime hotspots in Oldgate?", crime:6, money:4, fun:1, work:4, formal:6});

Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","No Crime","Efficient", {text:"We've been focusing on crime, and look there's hardly any!", crime:2, money:4, fun:6, work:6, formal:4});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","No Crime","Modest", {text:"Isn't it safe here in Oldgate.", crime:1, money:6, fun:6, work:6, formal:5});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","No Crime","Friendly", {text:"Hi there! Can I help you?", crime:3, money:6, fun:6, work:6, formal:2});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","No Crime","Formal", {text:"Crime is at an all-time low.", crime:3, money:6, fun:6, work:6, formal:6});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","No Crime","Sympathic", {text:"You say that you've been burgled. That's terrible given how little crime there is!", crime: 1, money:6, fun:4, formal:4 });
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","No Crime","Informal", {text:"Isn't it quiet?", crime:2, money:7, work:7, formal:3});
Firstline.addLine(["watch","wardmoot","witanmoot"],"Crime","No Crime","Aloof", {text: "Twelve o'clock and all's well!", crime:1, money:6, fun:3, formal:8});

Firstline.addLine(["tell","campaign","wardmoot"],"Money","Lower Poll Tax","Efficient", {text:"Oldgate's wardmoot is doing a great job, so I'll be working to reduce the Poll Tax this year", crime:3, money:10, fun:6, work:4});
Firstline.addLine(["tell","campaign","wardmoot"],"Money","Lower Poll Tax","Modest", {text:"I think %d should be able to reduce the poll tax this coming year.", crime:3, money:8, formal:7});
Firstline.addLine(["tell","campaign","wardmoot"],"Money","Lower Poll Tax","Sympathic", {text:"I know you're paying too much poll tax. I'll work to get your tax reduced in the coming year.", crime:3, money:10, formal:5});

Firstline.addLine(["tell","campaign","wardmoot"],"Money","Gax's Orphanage","Friendly", {text:"I think Gax the Heretic is doing a great job with his orphanage. I'll be supporting his efforts.", crime:3, money:3, fun:6, work:6});
Firstline.addLine(["tell","campaign","wardmoot"],"Money","Gax's Orphanage","Formal", {text:"Gax the Heretic shouldn't running his orphanage since he's supposed to be following the teaching of Hahn.", crime:3, money:6, fun:4, work:6, formal:8});

Firstline.addLine(["work","tell","campaign","wardmoot"],"Money","Discount","Efficient", {text:"If you vote for me I can give you a discount on painting %a.", money:7, fun:3, work:7});
Firstline.addLine(["work","tell","campaign","wardmoot"],"Money","Discount","Modest", {text:"A discount - well if you think you need it, I'll see what I can do.", money:6, work:6, formal:4});
Firstline.addLine(["work","tell","campaign","wardmoot"],"Money","Discount","Friendly", {text:"A discount for you, but of course!", money:7, work:7, formal:3});
Firstline.addLine(["work","tell","campaign","wardmoot"],"Money","Discount","Formal", {text:"If you vote for me I can give you a discount on painting %a - let's write that into the contract.", money:8, fun:3, work:7, formal:9});
Firstline.addLine(["work","tell","campaign","wardmoot"],"Money","Discount","Sympathic", {text:"Hmm, you do look like you need a discount on painting %a.", money:9, work:6, formal:3});
Firstline.addLine(["work","tell","campaign","wardmoot"],"Money","Discount","Informal", {text:"I'm sure we can come to an arrangement on price!", money:6, fun:4, work:7, formal:4});
Firstline.addLine(["work","tell","campaign","wardmoot"],"Money","Discount","Aloof", {text:"A discount? I'm not sure about that. My work is pretty good.", money:3, fun:3, work:8, formal:7});
Firstline.addLine(["work","tell","campaign","wardmoot"],"Money","Discount","Proud", {text:"I don't need to give discounts.", money:2, fun:3, work:8, formal:9});
Firstline.addLine(["work","tell","campaign","wardmoot"],"Money","Discount","Persuasive", {text:"Will a discount on painting %a clinch the deal?",money:7, fun:3, work:7});

Firstline.addLine(["tell","campaign","watch","wardmoot","witanmoot"],"Duty","Councillor","Efficient", {text:"I'll work to be the best councillor Oldgate has ever had.", crime:6, money:3, fun:3, work:7, formal:6});
Firstline.addLine(["tell","campaign","watch","wardmoot","witanmoot"],"Duty","Councillor","Modest", {text:"Well if you'll have me, I'll try to do my best as a councillor, but I'm not sure how well I'll do.", work:3, formal:2});
Firstline.addLine(["tell","campaign","watch","wardmoot","witanmoot"],"Duty","Councillor","Friendly", {text:"I'll represent us in the wardmoot and make sure we do well.", crime:6, money:4, fun:4, work:6, formal:2});
Firstline.addLine(["tell","campaign","watch","wardmoot","witanmoot"],"Duty","Councillor","Formal", {text:"I'm standing for councillor of Oldgate ward. Can I count on your support?", fun:3, work:5, formal:9});
Firstline.addLine(["tell","campaign","watch","wardmoot","witanmoot"],"Duty","Councillor","Sympathic", {text:"I'm planning to be your councillor next year. How can I help?", money:4, fun:6, work:6, formal:3});
Firstline.addLine(["tell","campaign","watch","wardmoot","witanmoot"],"Duty","Councillor","Informal", {text:"Yo! Let me be your councillor next year and I'll be there for you.", crime:4, money:4, fun:7, formal:9});
Firstline.addLine(["tell","campaign","watch","wardmoot","witanmoot"],"Duty","Councillor","Aloof", {text:"I'm up for election next year.", formal:7});
Firstline.addLine(["tell","campaign","watch","wardmoot","witanmoot"],"Duty","Councillor","Proud", {text:"I'm proud of Oldgate and would like to be a councillor next year to promote our ward.", money:4, fun:4, work:6, formal:7});
Firstline.addLine(["tell","campaign","watch","wardmoot","witanmoot"],"Duty","Councillor","Persuasive", {text:"I'm standing for election to the Oldgate wardmoot. Will you help me?", money:4, work:4, formal:8});

Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Walls", "Formal",{text:"The city faces many foes. We need to improve the city wall next to us in Oldgate.", crime:6, money:2, fun:2, work:6, formal:8, invasion:10});
Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Walls", "Informal",{text:"Build up the wall.", crime:6, money:2, fun:2, work:6, formal:2, invasion:10});

Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Save Money", "Efficient",{text:"I'll work to ensure that our taxes are well spent and not overspent", money:6, crime:4, plague:4, invasion:4, fire:4, famine:4});
Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Save Money", "Formal",{text:"We're being overtaxed, we need to get our taxes reduced.", money:10, fun:6, work:6, formal:7, crime:2, plague:2, invasion:2, famine:2, fire:2});
Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Save Money", "Informal",{text:"More money for us will sort out our problems", crime:3, money:9, fun:6, work:6, formal:2, plague:2, invasion:2, famine:2, fire:2});

Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Civic Ball", "Formal",{text:"If elected I pledge to ensure that the Civic Ball will be one of the best ever!", money:3, fun:10, work:3, formal:6, plague:0, invasion:0, famine:0});
Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Civic Ball", "Informal",{text:"Let's have a party!", crime:4, money:3, fun:10, work:3, formal:1,  plague:0, invasion:0, famine:0});

Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Sweet Water", "Formal",{text:"If elected I pledge that the Sweet Water project will be implemented.", money:0, work:3, formal:6, plague:10, famine:6, invasion:1, fire:6, crime:2});
Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Sweet Water", "Informal",{text:"We need to sort out the out-houses.", money:0, work:3, formal:3, plague:10, famine:6, invasion:1, fire:6, crime:2 });
Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Sweet Water", "Cancel",{text:"The expenditure on the sweet water is a waste of money. The Torians will never allow it through their scared lake!", money:8, plague:2, famine:4, invasion:6, fire:2});

Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Witanmoot Guard", "Expand",{text:"If elected I pledge to expand the witanmoot guard so they can assist the ward watches.", money:1, invasion:8, fire:6, crime:10});
Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Witanmoot Guard", "Reduce",{text:"If elected I pledge to reduce the number of the witanmoot guard and hence our poll tax.", money:10, invasion:1, fire:4, crime:0 });
Firstline.addLine(["tell","campaign","wardmoot","sherrif","witanmoot"],"City Politics","Witanmoot Guard", "Abolish",{text:"The Witanmoot Guard are corrupt enforcers of the Lord of the City. They must be abolished", money:10, formal:4, invasion:0, fire:2, crime:1, fun:10});

Firstline.addLine(["tell","campaign","watch","wardmoot"],"Crime","Filthy","Efficient", {text:"Oldgate is filthy. I'll be working to get toshers to clean up our ward.", crime:6, money:4, fun:4, work:6, formal:6, plague:7});
Firstline.addLine(["tell","campaign","watch","wardmoot"],"Crime","Filthy","Proud", {text:"Oldgate is filthy. I'll be working to clean up our ward.", crime:6, money:4, fun:4, work:6, formal:6, plague:7});

Firstline.penceOf = function(subject) {
    switch(subject) {
        case "Sixpence": return 6;
        case "One Shilling": return 12;
        case "Two Shillings": return 24;
        case "Five Shillings": return 60;
        case "Ten Shillings": return 120;
        case "One Pound": return 240;
        default: return undefined;
    }
};

Firstline.addLine(["tell","campaign","smear"],"Bribe","Sixpence","Openly", {text:"If you vote for me, I'll give you sixpence", crime:4, money:6, fun:4, work:4, formal:3});
Firstline.addLine(["tell","campaign","smear"],"Bribe","Sixpence","Quietly", {text:"Just between you and me, I will give you sixpence for your vote.", crime:4, money:6, fun:4, work:4, formal:7});
Firstline.addLine(["tell","campaign","smear"],"Bribe","One Shilling","Openly", {text:"If you vote for me, I'll give you a shilling", crime:4, money:7, fun:5, work:4, formal:3});
Firstline.addLine(["tell","campaign","smear"],"Bribe","One Shilling","Quietly", {text:"Just between you and me, I will give you a shilling for your vote.", crime:4, money:7, fun:5, work:4, formal:7});
Firstline.addLine(["tell","campaign","smear"],"Bribe","Two Shillings","Openly", {text:"If you vote for me, I'll give you two shillings", crime:4, money:8, fun:6, work:4, formal:3});
Firstline.addLine(["tell","campaign","smear"],"Bribe","Two Shillings","Quietly", {text:"Just between you and me, I will give you two shillings for your vote.", crime:4, money:8, fun:6, work:4, formal:7});
Firstline.addLine(["tell","campaign","smear"],"Bribe","Five Shillings","Openly", {text:"If you vote for me, I'll give you five shillings", crime:4, money:9, fun:8, work:4, formal:3});
Firstline.addLine(["tell","campaign","smear"],"Bribe","Five Shillings","Quietly", {text:"Just between you and me, I will give you five shillings for your vote.", crime:4, money:9, fun:8, work:4, formal:7});
Firstline.addLine(["tell","campaign","smear"],"Bribe","Ten Shillings","Openly", {text:"If you vote for me, I'll give you ten shillings", crime:3, money:10, fun:5, work:4, formal:3});
Firstline.addLine(["tell","campaign","smear"],"Bribe","Ten Shillings","Quietly", {text:"Just between you and me, I will give you ten shillings for your vote.", crime:3, money:10, fun:5, work:4, formal:7});
Firstline.addLine(["tell","campaign","smear"],"Bribe","One Pound","Openly", {text:"If you vote for me, I'll give you one pound", crime:0, money:10, fun:1, work:4, formal:3});
Firstline.addLine(["tell","campaign","smear"],"Bribe","One Pound","Quietly", {text:"Just between you and me, I will give you one pound for your vote.", crime:0, money:10, fun:1, work:4, formal:7});

//All 'undefined' firstlines must come last, so that we know all the valid reasons and hence add to the all of them
Firstline.addLine(undefined,"Gossip","Neighbour","whisper",{text: "Have you what %n is up to?", money:6, fun:4, work:3 });

//For testing via QUnit
if (typeof module !== "undefined" && module.exports) {
    exports.Firstline = Firstline;
}