/*!
 * © 2013, 2016 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */

/* globals ROT, Elector, femalenames, malenames, surnames, Address, Firstline, Chronicle, Faction, ReserveSherrifs */
var Months = ["Midwinter", "Skelern", "Galemir", "Pipetal", "Jasmarill", "Heliora", "Menderal", "Setheran", "Arvinoth", "Lorinfall", "Kryll", "Ternost", "Kruthos"];

var Left = ["in 12 months time","in 12 months time","in 11 months time","in 10 months time", "in 9 months time", "in 8 months time","in 7 months time","in 6 months time", "in 5 months time","in 4 months time","in 3 months time","in 2 months time","next month","at the end of this month"];

var Game =  {
    _currentScreen: null,
    _player: null,
    _electorate: [],
    _alderman: false,
    _aldermanIndex: 1,
    _councillor: false,
    _voting: false,
    _year: 137,
    _month: 0,
    _news: [],
    _innovated: false,
    _money: 120, /* pence */
    _factors: { crime:1, money:1, plague:1, invasion:1, famine:1, fire:1 },
    _smearCount: 0,
    _upsetAlderman: false,
    _lordOfCity: "Raphael Fitz-Simmons",
    _sherrifs: [{index:undefined, name:"Kennet Maxil"},{index: undefined, name:"Alan Griffiths"}],
    _estimatedVote: 1,

    captureException: function(f,arg) {
        try {
            f(arg);
        } catch(ex) {
            var exdiv = document.getElementById("exception");
            exdiv.className = exdiv.className.replace(/exhidden/,"exvis");
            var element = document.getElementById("exception.text");
            var text;
            if (ex instanceof Error) {
                text = ex.name+" : "+ex.message;
            } else {
                text = ex.toString();
            }
            text = text + '\n' + (ex.stack || ex.stacktrace || '');

            // Remove the chunk in the stack trace between @ and /js/getelected/
            text = text.replace(/@.*\/js\/getelected\//g," in ");
            // And version field too
            text = text.replace(/\?ver=[0-9a-fA-F]*:/g,":");

            var newText = document.createTextNode(text);
            if (element.hasChildNodes()) {
                element.replaceChild(newText, element.firstChild);
            } else {
                element.appendChild(newText);
            }
        }
    },
    captureExceptionFunction: function(f, arg) {
        return function() {
            Game.captureException(f,arg);
        };
    },
    captureExceptionEventHandler: function(f) {
        return function(e) {
            Game.captureException(f,e);
        };
    },
    switchScreen: function(screen) {
        // If we had a screen before, notify it that we exited
        if (this._currentScreen !== null) {
            this._currentScreen.exit();
        }
        // Mark all the divs that were gevis as gehidden
        var div;
        var divs = document.getElementsByClassName("gevis");
        for(var d=0; d<divs.length; d++) {
            div = divs[d];
            div.className = div.className.replace(/gevis/,"gehidden");
        }
        // Mark the screen's div as gevis
        div = document.getElementById(screen.name);
        div.className = div.className.replace(/gehidden/,"gevis");

        // Update our current screen, notify it we entered
        // and then render it
        this._currentScreen = screen;
        if (this._currentScreen !== null) {
            // Enter and render the current screen
            this.captureException(function(self){
                self._currentScreen.enter();
                self._currentScreen.render();
                self.renderExtra();
            },this);
        }
    },
    renderExtra: function() {
        var netPence = Game._money % 12;
        var shillings = ((Game._money - netPence)/12) % 20;
        var pounds = ((Game._money - netPence - shillings*12)/240);
        if (netPence === 0 && shillings === 0) {
            Game.setIdText("money.total","£"+pounds);
        } else if (netPence === 0) {
            Game.setIdText("money.total","£"+pounds+" "+shillings+"/-");
        } else {
            Game.setIdText("money.total","£"+pounds+" "+shillings+"/"+netPence);
        }
        if (Game._estimatedVote === 1) {
            Game.setIdText("money.vote","no one except yourself");
        } else {
            Game.setIdText("money.vote",Game._estimatedVote+" people");
        }
        Game.setIdText("money.time",Left[Game._month+1]);
    },

    addElector: function(name,surname,gender,address,guild) {
        var genders = ["F", "M"];
        var gendernames = {F: femalenames, M: malenames};

        surname = surname || surnames.random();
        gender = gender || genders.random();
        if (name === undefined) {
            do {
                name = gendernames[gender].random();
            } while(this._electorate[name+" "+surname] !== undefined);
        }

        if (this._electorate.length < 20) { guild = 1; } // Ensure the first few electors are all guildmembers since they could be councillors too.
        
        this._electorate.push(new Elector(this._electorate.length,name+" "+surname, gender, 0.7, address, guild));
        this._electorate[name+" "+surname] = this._electorate.length-1;
    },

    /* jshint sub:true */
    gameInit: function(self) {
        // Set up the electorate
        self._electorate[1] = new Elector(1,"Samatha Cox","F",1.0,Address.getAddresses()[132],"Apothecaries");
        self._electorate[2] = new Elector(2,"Duncan Barnett","M",1.0,Address.getAddresses()[44],"Mercers");
        self._electorate[3] = new Elector(3,"John Madigan","M",1.0,Address.getAddresses()[67],"Plasterers");
        self._electorate[4] = new Elector(4,"Emily Martel","F",1.0,Address.getAddresses()[98],"Skinners & Leathersellers");

        var addr;
        do { addr = Address.random(); } while(addr.getInhabitants().length !== 0);
        self._player = new Elector(0,"YOU","?",1.0,addr,"Painters");
        self._electorate[0] = self._player;

        self._electorate["YOU"] = 0;
        self._electorate["Samantha Cox"] = 1;
        self._electorate["Duncan Barnett"] = 2;
        self._electorate["John Madigan"] = 3;
        self._electorate["Emily Martel"] = 4;

        // Other names to skip to avoid confusion
        Faction.forEach(function(f){
            self._electorate[f.getLeader()] = -1;
        });
        self._electorate[self._lordOfCity] = -1;
        self._electorate[self._sherrifs[0].name] = -1;
        self._electorate[self._sherrifs[1].name] = -1;
        ReserveSherrifs.forEach(function(rs){
            self._electorate[rs.name] = -1;
        });
        
        // Set up the alderman's status vis-a-vis the sherrifs
        Game._sherrifIndexes = []; // No one in the ward is a sherrif at the start
        Game._sherrifVotes = 1; 

        
        // Give the alderman a faction
        self._electorate[self._aldermanIndex].setFaction();
        
        // Add one person to each household initially (in random order since we need the other councillors at the front (skip those we prefilled above)
        var cloneaddr = Address.getAddresses().concat().randomize();
        cloneaddr.forEach(function(v) {
            if (v.getInhabitants().length === 0) {
                self.addElector(undefined,undefined,undefined,v,0.8); // Householders very likely to be guildmembers
            }
        });

        // Marry off 80% of the addresses
        Address.forEach(function(v){
            var surname;
            if (ROT.RNG.getPercentage() <= 80 && v !== addr) {
                surname = v.getHouseholder().getSurname();
                var guild = 0.5;
                self.addElector(undefined,surname,(v.getHouseholder().getGender() === "M" ? "F" : "M"),v,guild);  
                // 50% of those have another family member (with the same surname) that can vote - this repeats
                while(ROT.RNG.getPercentage() <= 50) {
                    guild /= 2;
                    self.addElector(undefined,surname,undefined,v,guild);
                }
            }
        });

        // And then 50% of all address have another member (with a random surname) that can also vote - this repeats)
        Address.forEach(function(v){
            var guild = 0.35;
            while(ROT.RNG.getPercentage() <= 50) {
                self.addElector(undefined,undefined,undefined,v,guild);
                guild /=2 ;
            }
        });
        
        // Plus a bonus member for the player's address (to avoid an infinite loop in selecting who to talk to
        self.addElector(undefined,undefined,undefined,addr,0.5);

        // Set up initial preferences
        Address.forEach(function(v){
            var householderPreference = null;
            if (v.getHouseholder().getIndex() < 14) {
                householderPreference = v.getHouseholder().getIndex(); // Alderman, councillor and the player vote for self
            }
            v.getHouseholder().setInitialPreference(householderPreference,self._electorate);
            householderPreference = v.getHouseholder().getFirstPreference();
            v.forEachInhabitant(function(w){
                if (w !== v.getHouseholder()) {
                    if (ROT.RNG.getPercentage() <= 80) {
                        w.setInitialPreference(householderPreference,self._electorate);
                    } else {
                        w.setInitialPreference(null,self._electorate);
                    }
                }
            });
        });

        // set up initial chronicle
        Chronicle.write(self._news,self._year,Months[self._month]);
    },

    init: function(game) {
        // Any necessary initialization will go here.
        // Create a helper function for binding to an event
        // and making it send it to the screen
        var bindEventToScreen = function(event) {
            window.addEventListener(event, function(e) {
                // When an event is received, send it to the
                // screen if there is one
                if (game._currentScreen !== null) {
                    // Send the event type and data to the screen
                    game.captureException(function(self){
                        self._currentScreen.handleInput(event, e);
                        self._currentScreen.render();
                        self.renderExtra();
                    },game);
                }
            });
        };

        Game.captureException(Game.gameInit,game);

        // Bind keyboard input events
        bindEventToScreen('keydown');
        bindEventToScreen('keypress');
    },

    nextMonth: function() {
        this._month++;
        if (this._month === 13) {
            this._month = 0;
            this.nextYear();
        }
        // Push the factors back towards 1
        for(var f in Game._factors) {
            if (Game._factors.hasOwnProperty(f)) {
                Game._factors[f] = (Game._factors[f] * 11 + 1)/12;
            }
        }
        
        // Push the smear back towards zero (but slowly)
        Game._smearCount *= 0.99;
        
        // If the alderman is upset, visit a random household and reverse the smear action
        if (Game._upsetAlderman) {
            Address.random().getInhabitants().forEach(function(v){
                if (v.getIndex() !== 0 && v.getFirstPreference() === 0) {
                    v.diminishPreference(Game._electorate,0.75);
                }
            });
        }
    },

    nextYear: function() {
        this._year++;
        Address.forEach(function(a){
            a.nextYear();
        });
        Game.retaliate(Game._smearCount);
    },
    implementPlan: function(reason,topic,view,attitude,person,score) {
        Chronicle.write(Game._news,Game._year,Months[Game._month]);
        var changedVote = reason.implement(topic,view,person,score);
        Faction.forEach(function(f) {
            f.rateLine(Firstline.getLine(reason,topic,view,attitude));
            f.makeNews();
        });
        Faction.recruitAlderman(Game._electorate[Game._aldermanIndex]);
        if (topic === "Bribe") {
            Game._money -= Firstline.penceOf(view);
            this.renderExtra();
        }
        return changedVote;
    },
    implementWorkHard: function(topic,view,person,score) {
        var oldPreference = person.getFirstPreference();
        person.getAddress().forEachInhabitant(function(v){
            v.improvePreference(Game._electorate,score/100*person.getSimilarity(v));
        });

        // score also gives the change that the work is actually carried out
        if (ROT.RNG.getPercentage() <= score) {
            person.getAddress().renovate();
            Game._money += (view === "Discount" ? 12 : 60);
            if (person.getAddress().getHouseState().getCondition() < -40 && !Game._innovated) {
                var adj = "well-established ";
                if (Game._year < 140) {
                    adj = "young new ";
                }
                Game._news.push('In oldgate many patrons are praising the work of a '+
                        adj+'painter and decorator whose new "double-walled" style has allowed them '+
                        'to enjoy their newly dramatically painted rooms while remaining warm in winter '+
                        'without the need for tapestries to insulate the walls. '+
                        person.getName()+' commented "I\'m so proud of '+person.getAddress().getAddress()+' now."');
                Game._innovated = true;
            }
        }
        return (oldPreference !== person.getFirstPreference());
    },
    implementPlayHard: function(topic,view,person,score) {
        return person.improvePreference(Game._electorate,score/100);
    },
    implementTell: function(topic,view,person,score) {
        person.getAddress().forEachInhabitant(function(v){
            if (v.getFirstPreference() === 0) {
                v.setVoteChance(score/100);
            }
        });
        return false;
    },
    implementCampaign: function(topic,view,person,score) {
        var oldPreference = person.getFirstPreference();
        person.getAddress().forEachInhabitant(function(v){
            v.improvePreference(Game._electorate,score/100*person.getSimilarity(v));
        });
        return (oldPreference !== person.getFirstPreference());
    },
    implementSmear: function(topic,view,person,score) {
        var odds;
        if (Game._councillor) {
            odds = 75;
        } else {
            odds = 25;
        }
        // the alderman's supporters have a chance to have their preference reduced
        person.getAddress().getInhabitants().forEach(function(v){
            if (v.getIndex() !== Game._aldermanIndex && v.getFirstPreference() === Game._aldermanIndex && ROT.RNG.getPercentage() <= odds) {
                v.diminishPreference(Game._electorate,score/100*person.getSimilarity(v),Game._aldermanIndex);
                Game._smearCount += 5;
                Game._electorate[Game._aldermanIndex].getFaction().smear(5);
            }
        });
        if (ROT.RNG.getPercentage() < Game._smearCount) {
            Game.retaliate(100); // The alderman is upset and certain to retaliate
        }
    },
    implementWardMoot: function(topic,view,person,score) {
        var oldPreference = person.getFirstPreference();
        // If not on the wardmoot, reduce score by a factor of 5
        if (!Game._alderman && !Game._councillor) { score = score/5; }
        person.getAddress().forEachInhabitant(function(v){
            v.improvePreference(Game._electorate,score/100*person.getSimilarity(v));
        });
        return (oldPreference !== person.getFirstPreference());
    },
    implementWatch: function(topic,view,person,score) {
        var oldPreference = person.getFirstPreference();
        // If not on the wardmoot, reduce score by a factor of 5
        if (!Game._alderman && !Game._councillor) { score = score/5; }
        person.getAddress().forEachInhabitant(function(v){
            v.improvePreference(Game._electorate,score/100*person.getSimilarity(v));
        });
        return (oldPreference !== person.getFirstPreference());
    },
    implementWitanmoot: function(topic,view,person,score) {
        var oldPreference = person.getFirstPreference();
                // If not on the wardmoot, reduce score by a factor of 5
        if (!Game._alderman && !Game._councillor) { score = score/5; }
        person.getAddress().forEachInhabitant(function(v){
            if (v.getVoted()) {
                v.improvePreference(Game._electorate,score/100*person.getSimilarity(v));
            }
        });
        return (oldPreference !== person.getFirstPreference());
    },
    implementPromote: function(topic,view,person,score,factor) {
        // If not on the wardmoot, reduce score by a factor of 10
        if (!Game._alderman && !Game._councillor) { score = score/10; }
        Game._sherrifVotes += score/100;
        // And makes the alderman less likely to remain/become upset next year
        Game._smearCount -= 3;
        Game._electorate[Game._aldermanIndex].getFaction().promote(3);
        // And possibly stop being upset
        if (Game._upsetAlderman && ROT.RNG.getPercentage() > Game._smearCount) {
            Game._upsetAlderman = false;
        }
        return false;
    },
    retaliate: function(odds) {
        var oldUpset = Game._upsetAlderman;
        if (ROT.RNG.getPercentage() <= odds) {
            Game._upsetAlderman = true;
            if (oldUpset === false) {
                Game._news.push("Alderman "+Game._electorate[Game._aldermanIndex].getName()+" has complained to the Witanmoot about the rumours swirling around Oldgate. 'If I find out who is responsible, I will do everything I can do to ensure they're not in my wardmoot!'");
            }
        } else {
            Game._upsetAlderman = false;
        }
    },
    setIdText: function(id, text, tooltip, onclick) {
        var element = document.getElementById(id);
        if (!element) {
            throw new Error("No document element with the id of '"+id+"'");
        }
        var newText = document.createTextNode(text);
        if (tooltip) {
            element.title = tooltip;
        }
        element.onclick = onclick; // Will set to undefined by default
        if (element.hasChildNodes()) {
            element.replaceChild(newText, element.firstChild);
        } else {
            element.appendChild(newText);
        }
    },
    setIdHTML: function(id, html, tooltip, onclick) {
        var element = document.getElementById(id);
        if (!element) {
            throw new Error("No document element with the id of '"+id+"'");
        }
        element.innerHTML = html;
        if (tooltip) {
            element.title = tooltip;
        }
        element.onclick = onclick; // Will set to undefined by default
    },
    clearNode: function(id) {
        var element = document.getElementById(id);
        while (element.hasChildNodes()) {
            element.removeChild(element.lastChild);
        }
    },
    addElementToNode: function(type,id,newid,text,style,onclick) {
        text = text || "..."; // Ensure we have some kind of text
        var newElement = document.createElement(type);
        if (newid) {
            newElement.id = newid;
        }
        if (style) {
            newElement.className = style;
        }
        newElement.onclick = onclick; // Will set to undefined by default
        newElement.innerHTML = text;
        document.getElementById(id).appendChild(newElement);
    },
    clearDiv: function(id) {
        this.clearNode(id);
    },
    addParagraphToDiv: function(id,text) {
        Game.addElementToNode("p",id,undefined,text);
    },
    clearList: function(id) {
        this.clearNode(id);
    },
    addItemToList: function(id,newid,text,onclick) {
        Game.addElementToNode("li",id,newid,text,undefined,onclick);
    },
    nextChar: function(c) {
        var alphabet = "ABCDEFGHIJKLMNOPQR123456789";
        return alphabet.charAt(alphabet.indexOf(c)+1);
    }
};

window.onload = function() {
    var startscreen = document.getElementById("screen.start");
    var elt;
    // Check if rot.js can work on this browser
    if (!ROT.isSupported()) {
        elt = document.createElement("p");
        elt.innerHTML = "The rot.js library isn't supported by your browser.";
        startscreen.appendChild(elt);
        //alert("The rot.js library isn't supported by your browser.");
    // Check if CSS pointer-events is supported on this browser
    } else {
        if (!('pointerEvents' in startscreen.style)) {
           elt = document.createElement("p");
           elt.innerHTML = "Your browser doesn't seem to support the CSS pointer-events property - you'll probably be forced to chose the random address option.";
           startscreen.appendChild(elt);
        }
        // Initialize the game
        Game.captureException(Address.init); // Addresses first - since Game.init needs the list of address (which we compute from the html)
        Game.captureException(Game.init,Game);
        Game.captureException(Address.displayData(Address.random())); // Need to have Game.init'ed to get all the inhabitants set up
        Game.captureException(Address.redrawCanvas); // Give the address a chance to draw the overlay of houses - must be after Address.init
        window.onresize = Game.captureExceptionFunction(Address.redrawCanvas);
        // Load the start screen
        Game.switchScreen(Game.Screen.startScreen);
    }
};
