//
// © 2016 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//

/* globals Game: true, ROT, Months, Firstline, Reason, Address: true */

Game.Screen = {};

// Define our initial start screen
Game.Screen.startScreen = {
    name: "screen.start",
    enter: function() { },
    exit: function() { },
    render: function() {
        document.getElementById("start.next").onclick = function() { Game.switchScreen(Game.Screen.electionScreen); };
    },
    handleInput: function(inputType, inputData) {
        // When [Enter] is pressed, go to the play screen
        if (inputType === 'keydown') {
            if (inputData.keyCode === ROT.VK_RETURN) {
                Game.switchScreen(Game.Screen.electionScreen);
            }
        }
    }
};

// Define our playing screen
Game.Screen.electionScreen = {
    name: "screen.election",
    enter: function() {
        Game.Screen.electionScreen._lastCount = 0;
        // Decide if people voted this time - and reset the alderman and councillor flags
        for(var i=0; i<Game._electorate.length; i++) {
            Game._electorate[i].setAlderman(false);
            Game._electorate[i].setCouncillor(false);
            Game._electorate[i].decideIfVoted();
            Game._electorate[i].decideIfSecretlyVoted();
        }
    },
    exit: function()  { },
    plural: function(n,s,pad,align) {
        var padded = n.toString();
        if (pad !== undefined) {
            n = ("         "+n).slice(-pad);
        }
        if (n === 1 ) {
            if (align === true) {
                return padded+" "+s+" ";
            } else {
                return padded+" "+s;
            }
        } else {
            return padded+" "+s+"s";
        }
    },
    render: function() {
        Game.setIdText("election.month",Months[Game._month]);
        Game.setIdText("election.year",Game._year);
        // Count the votes
        var votes = [0]; // Player starts with no votes
        
        for(var v=0; v<Game._electorate.length; v++) {
            Game._electorate[v].saveFirstPreference();
            if (Game._electorate[v].getVoted() ) {
                var vote = Game._electorate[v].getFirstPreference();
                if (votes[vote] === undefined) {
                    votes[vote] = 1;
                } else {
                    votes[vote]++;
                }
            }
        }
        
        // Work out the ordered nominees
        var nominees = [0];
        for(var n=1; n<votes.length; n++) {
            if (votes[n] !== undefined) {
                if (votes[n] === 1) {
                    nominees.push(n); // Let the player get the benefit of the doubt if they only have one vote
                } else if (votes[n] <= votes[nominees[nominees.length-1]]) {
                    nominees.push(n);
                } else if (votes[n] > votes[nominees[0]]) {
                    // New leader
                    nominees.unshift(n);
                } else {
                    // Somewhere in the middle
                    for(var nomIndex = 1; nomIndex < nominees.length; nomIndex++) {
                        if (votes[n] >= votes[nominees[nomIndex]]) {
                            nominees = nominees.slice(0,nomIndex).concat(n).concat(nominees.slice(nomIndex,nominees.length));
                            break;
                        } 
                    }
                }
            }
        }

        // And display the results (resetting the flags in the electorate as we go
        Game._alderman = (nominees[0] === 0);
        var oldAldermanIndex = Game._aldermanIndex;
        Game._aldermanIndex = nominees[0];
        Game._electorate[Game._aldermanIndex].setAlderman(true);

        Game._councillor = false;
        Game.setIdText("election.alderman.votes",Game.Screen.electionScreen.plural(votes[nominees[0]],"Vote",3,true));
        Game.setIdText("election.alderman",Game._electorate[nominees[0]].getName());
        for (var c=1; c<13; c++) {
            if (c < nominees.length) {
                Game.setIdText("election.councillor"+c+".votes",Game.Screen.electionScreen.plural(votes[nominees[c]],"Vote",3,true));
                Game.setIdText("election.councillor"+c,Game._electorate[nominees[c]].getName());
                if (nominees[c] === 0) {
                    Game._councillor = true;
                }
                Game._electorate[nominees[c]].setCouncillor(true);
            }
        }
        
        Game._nominees = nominees; // Save the ordered nominee list
        Game._estimatedVote = votes[0]; // And the player's votes
        if (Game._alderman) {
            Game.setIdText("election.result","You've made it! - You're the alderman!");
            Game.setIdText("election.next",
                    "Press [Enter] to finish",
                    undefined,
                    function() { Game.switchScreen(Game.Screen.winScreen); }
            );
        } else {
            Game.setIdText("election.next",
                    "Press [Enter] to make your plans for the next year.",
                    undefined,
                    function() { Game.switchScreen(Game.Screen.planScreen); }
            );
            if (Game._councillor) {
                Game.setIdText("election.result","You're a councillor.");
            } else {
                Game.setIdText("election.result","Oh dear! With only "+Game.Screen.electionScreen.plural(votes[0],"vote")+" you don't have enough votes even to be a councillor.");
            }
        }
        if (oldAldermanIndex !== Game._aldermanIndex) {
            // The alderman has changed - it's a bit of fudge, but reset the game flags for the alderman
            Game._smearCount = 0;
            Game._upsetAlderman = false;
            Game._sherrifVotes = 1;
            // If the old alderman is illegible - they were promoted last year (so the election of a new alderman is not unexpected
            if (Game._electorate[oldAldermanIndex].isEligible()) {
                // And if the alderman has no faction, randomize one now
                if (Game._electorate[Game._aldermanIndex].getFaction() === undefined) {
                    Game._electorate[Game._aldermanIndex].setFaction();
                    Game._news.push("In a shock result, Oldgate has a new alderman, "+Game._electorate[Game._aldermanIndex].getName()+" after "+Game._electorate[oldAldermanIndex]+" was elected as sherrif last year. There was a mad scramble amongst the faction leaders to woo the new alderman. After several meetings, discussions and general over-the-table discussions, "+Game._electorate[Game._aldermanIndex].getName()+" annouced "+Game._electorate[Game._aldermanIndex].hir()+" membership of the "+Game._electorate[Game._aldermanIndex].getFaction().getLeader()+" faction.");
                } else {
                    Game._news.push(Game._electorate[oldAldermanIndex]+" has lost the aldermanship of Oldgate, being replaced by "+Game._electorate[Game._aldermanIndex].getName());
                }
            } else {
                if (Game._electorate[Game._aldermanIndex].getFaction() === undefined) {
                    Game._electorate[Game._aldermanIndex].setFaction();
                    Game._news.push("Oldgate has a new alderman, "+Game._electorate[Game._aldermanIndex].getName()+". There was a mad scramble amongst the faction leaders to woo the new alderman. After several meetings, discussions and general over-the-table discussions, "+Game._electorate[Game._aldermanIndex].getName()+" annouced "+Game._electorate[Game._aldermanIndex].hir()+" membership of the "+Game._electorate[Game._aldermanIndex].getFaction().getLeader()+" faction.");
                } else {
                    Game._news.push(Game._electorate[Game._aldermanIndex].getName()+" has been elected as the new Alderman for Oldgate after "+Game._electorate[oldAldermanIndex]+" was elected as sherrif last year.");
                }
            }
        }
    },
    handleInput: function(inputType, inputData) {
        if (inputType === 'keydown') {
            // If enter is pressed move onto planning
            if (inputData.keyCode === ROT.VK_RETURN) {
                if (Game._alderman) {
                    Game.switchScreen(Game.Screen.winScreen);
                } else {
                    Game.switchScreen(Game.Screen.planScreen);
                }
            }
        }    
    }
};
//Define the chronicle screen
Game.Screen.chronicleScreen = {
    name: "screen.chronicle",
    enter: function() { 
        Address.displayData(Address.random())();
        Address.activateMap(true);
        Game.setIdText("chronicle.next.address",Address.getSelected().getAddress());
        Address.selectedChangeListener = function(address) {
            Game.setIdText("chronicle.next.address",Address.getSelected().getAddress());
        };
        Address.selectedClickListener = function() {
            Game.switchScreen(Game.Screen.monthlyScreen);
        };
    },
    exit: function() { 
        Address.selectedChangeListener = undefined;
        Address.selectedClickListener = undefined;
        Address.activateMap(false);
        Game._news = [];
    },
    render: function() {
        // Render our prompt to the screen
        Game.setIdText("chronicle.year",Game._year);
        Game.setIdText("chronicle.month",Months[Game._month]);
        Game.clearDiv("screen.chronicle.contents");
        
        for(var i=0; i<Game._news.length; i++) {
            Game.addParagraphToDiv("screen.chronicle.contents",Game._news[i]);
        }
        
        document.getElementById("chronicle.next").onclick = function() { Game.switchScreen(Game.Screen.monthlyScreen); };
    },
    handleInput: function(inputType, inputData) {
        if (inputType === 'keydown') {
            // If enter is pressed move onto the next nonth
            if (inputData.keyCode === ROT.VK_RETURN) {
                Game.switchScreen(Game.Screen.monthlyScreen);
            }
        }    
    }
};
//Define our planning screen
Game.Screen.planScreen = {
    name: "screen.planning",
    _plan: undefined,
    _plans: undefined,
    enter: function() { Game.Screen.planScreen._plan = undefined; },
    exit: function() {
        Address.selectedChangeListener = undefined;
        Address.selectedClickListener = undefined;
        Address.activateMap(false);
        },
    actionButton: function(reason) {
        return function() {
            Game.Screen.planScreen._plan = reason;
            Game.captureException(Game.Screen.planScreen.render);
        };
    },
    render: function() {
        // Render our prompt to the screen
        Game.setIdText("planning.year",Game._year);
        Game.clearList("screen.options");
        var reasons = Reason.getReasons(Game._councillor);
        var selectedOption;
        var option = 0;
        var letter = 'A';
        Game.Screen.planScreen._plans = {};
        while(option < reasons.length) {
            if (Game.Screen.planScreen._plan === reasons[option]) {
                selectedOption = option;
            }
            Game.addItemToList("screen.options","screen.option"+option,reasons[option].getDescription(),Game.Screen.planScreen.actionButton(reasons[option]));
            Game.Screen.planScreen._plans[letter] = reasons[option];
            option++;
            letter = Game.nextChar(letter);
        }
        
        if (Game.Screen.planScreen._plan) {
            document.getElementById("screen.option"+selectedOption).className="selectedoption";
            if (Game._news.length > 0) {
                Game.setIdText("planning.next",
                        "Press [Enter] to put your plan into action and view the latest issue of the Chronicle.",
                        undefined,
                        Game.Screen.planScreen._nextMonth
                    );
            } else {
                Address.displayData(Address.random())();
                Address.activateMap(true);
                Address.selectedChangeListener = function(address) {
                    Game.setIdText("planning.next",
                            "Press [Enter] to put your plan into action and visit "+address.getAddress()+" or double-click the address you wish to visit (or press the letter of a plan to revise your plan).",
                            undefined,
                            Game.Screen.planScreen._nextMonth
                        );
                };
                Address.selectedChangeListener(Address.getSelected());
                Address.selectedClickListener = function() {
                    Game.Screen.planScreen._nextMonth();
                };
            }
        } else {
            Game.setIdText("planning.next","Press the letter of the plan you wish to implement",undefined,undefined);
            Address.selectedChangeListener = undefined;
            Address.selectedClickListener = undefined;
        }
    },
    _nextMonth: function() {
        Game.nextMonth();
        if (Game._news.length > 0) {
            Game.switchScreen(Game.Screen.chronicleScreen);
        } else {
            Game.switchScreen(Game.Screen.monthlyScreen);
        }
    },
    handleInput: function(inputType, inputData) {
        if (inputType === 'keydown') {
            if (inputData.keyCode === ROT.VK_RETURN && Game.Screen.planScreen._plan) {
                Game.Screen.planScreen._nextMonth();
            } else if (inputData.keyCode === ROT.VK_A) {
                Game.Screen.planScreen._plan = Game.Screen.planScreen._plans.A;
            } else if (inputData.keyCode === ROT.VK_B) {
                Game.Screen.planScreen._plan = Game.Screen.planScreen._plans.B;
            } else if (inputData.keyCode === ROT.VK_C) {
                Game.Screen.planScreen._plan = Game.Screen.planScreen._plans.C;
            } else if (inputData.keyCode === ROT.VK_D) {
                Game.Screen.planScreen._plan = Game.Screen.planScreen._plans.D;
            } else if (inputData.keyCode === ROT.VK_E) {
                Game.Screen.planScreen._plan = Game.Screen.planScreen._plans.E;
            } else if (inputData.keyCode === ROT.VK_F) {
                Game.Screen.planScreen._plan = Game.Screen.planScreen._plans.F;
            } else if (inputData.keyCode === ROT.VK_G) {
                Game.Screen.planScreen._plan = Game.Screen.planScreen._plans.G;
            } else if (inputData.keyCode === ROT.VK_H) {
                Game.Screen.planScreen._plan = Game.Screen.planScreen._plans.H;
            } else if (inputData.keyCode === ROT.VK_I) {
                Game.Screen.planScreen._plan = Game.Screen.planScreen._plans.I;
            }
            if (!Game._councillor && Game.Screen.planScreen._plan && Game.Screen.planScreen._plan.needsCouncillor()) {
                Game.Screen.planScreen._plan = undefined; // Reset - it's not a possible option at this time
            }
        }    
    }
};

Game.Screen.monthlyScreen = {
        name: "screen.monthly",
        _person: undefined,
        _address: undefined,
        _topic: undefined,
        _view: undefined,
        _attitude: undefined,
        _changedVote: undefined,
        _result: undefined,
        _implemented: false,
        _conversation: [],
        _choices: {},
        _alphabet: "ABCDEFGHIJKLMNOPQR123456789",
        _neighbour: undefined,
        _longReasonFormat: {
           person: function() { return Game.Screen.monthlyScreen._person.getName(); },
           address: function() { return Game.Screen.monthlyScreen._address.getAddress(); },
           alderman: function() { return Game._electorate[Game._aldermanIndex].getName(); },
           state: function() { return Game.Screen.monthlyScreen._address.getHouseState().getAdjective();},
           neighbour: function() { return Game.Screen.monthlyScreen._neighbour.getName(); },
           candidate: function() { return Game._electorate[Game.Screen.monthlyScreen._person.getVotedFor()].getName(); }
        },
        _extendFunctionWithRender: function(f) {
            return function() {
                f.apply(this, arguments);
                Game.captureException(Game.Screen.monthlyScreen.render);
                Game.captureException(Game.renderExtra);
            };
        },
        _setupTopic: function(topic) {
                return function() { 
                    Game.Screen.monthlyScreen._topic = topic;
                    Game.Screen.monthlyScreen._view = Firstline.getViews(Game.Screen.monthlyScreen._reason.getCode(),Game.Screen.monthlyScreen._topic)[0];
                    Game.Screen.monthlyScreen._attitude = Firstline.getAttitudes(Game.Screen.monthlyScreen._reason.getCode(),Game.Screen.monthlyScreen._topic,Game.Screen.monthlyScreen._view)[0];
                };
        },
        _setupSubject: function(subject) {
            return function() { 
                Game.Screen.monthlyScreen._view = subject; 
                Game.Screen.monthlyScreen._attitude = Firstline.getAttitudes(Game.Screen.monthlyScreen._reason.getCode(),Game.Screen.monthlyScreen._topic,Game.Screen.monthlyScreen._view)[0];
            };
        },
        _setupAttitude: function(att) {
            return function() { Game.Screen.monthlyScreen._attitude = att; };
        },
        enter: function() {
            var reasons = {};
            Reason.getReasons(Game._councillor).forEach(function(r) {
                reasons[r.getCode()] = 1;
            });
            reasons[Game.Screen.planScreen._plan.getCode()] = 10; // The plan is the most likely reason to meet
            
            // Can't promote if the alderman is not eligible - they're already promoted!
            if (!Game._electorate[Game._aldermanIndex].isEligible()) {
                reasons.promote = 0;
            }
            
            Game.Screen.monthlyScreen._reason = Reason.forCode(ROT.RNG.getWeightedValue(reasons));
            
            do { 
                Game.Screen.monthlyScreen._person = Address.getSelected().getInhabitants().random();
            } while(Game.Screen.monthlyScreen._person.getIndex() === 0); // Don't visit yourself
            
            // Choose a neighbour
            Game.Screen.monthlyScreen._neighbour = Game.Screen.monthlyScreen._person.getAddress().getNeighbours().random().getInhabitants().random();
            
            // Note: bribe is not the first topic or view, hence no need for the bribe logic here (phew!)
            Game.Screen.monthlyScreen._address = Game.Screen.monthlyScreen._person.getAddress();
            Game.Screen.monthlyScreen._topic = Firstline.getTopics(Game.Screen.monthlyScreen._reason.getCode())[0];
            Game.Screen.monthlyScreen._view = Firstline.getViews(Game.Screen.monthlyScreen._reason.getCode(),Game.Screen.monthlyScreen._topic)[0];
            Game.Screen.monthlyScreen._attitude = Firstline.getAttitudes(Game.Screen.monthlyScreen._reason.getCode(),Game.Screen.monthlyScreen._topic,Game.Screen.monthlyScreen._view)[0];
            Game.Screen.monthlyScreen._conversation = [];
            Game.Screen.monthlyScreen._changedVote = false;
            Game.Screen.monthlyScreen._result = undefined;
            Game.Screen.monthlyScreen._implemented = false;
        },
        exit: function() { 
            Address.activateMap(false);
            Address.selectedChangeListener = undefined;
            Address.selectedClickListener = undefined;
        },
        render: function() {
            // Render our prompt to the screen
            Game.setIdText("monthly.month",Months[Game._month]);
            Game.setIdText("monthly.year",Game._year);

            String.format.map.p = "person";
            String.format.map.a = "address";
            String.format.map.d = "alderman";
            String.format.map.h = "state";
            String.format.map.n = "neighbour";
            String.format.map.c = "candidate";
            Game.setIdText("monthly.meet",Game.Screen.monthlyScreen._reason.getLongReason().format(Game.Screen.monthlyScreen._longReasonFormat,Game.Screen.monthlyScreen._longReasonFormat,Game.Screen.monthlyScreen._longReasonFormat));
            Game.setIdText("monthly.description",Game.Screen.monthlyScreen._person.description(Game._electorate,Game._factors));
            
            for(r=1; r<4; r++) {
                for(c=1; c<4; c++) {
                    Game.setIdText("monthly.cell1"+r+c,"");
                    Game.setIdText("monthly.cell2"+r+c,"");
                    Game.setIdText("monthly.cell3"+r+c,"");
                }
            }

            var letter='A';
            var r=1, c=1;
            var topics = Firstline.getTopics(Game.Screen.monthlyScreen._reason.getCode());
            Game.Screen.monthlyScreen._choices = [];
            for(var i=0; i<topics.length; i++) {
                var topic = topics[i];
                if (topic !== "Bribe" || Game._money > 0) {
                    Game.setIdText("monthly.cell1"+r+c,letter+". "+topic);
                    Game.Screen.monthlyScreen._choices[letter] = Game.Screen.monthlyScreen._setupTopic(topic,true);
                    letter = Game.nextChar(letter);
                
                    if (topic === Game.Screen.monthlyScreen._topic) {
                        document.getElementById("monthly.cell1"+r+c).className="selectedoption";
                    } else {
                        document.getElementById("monthly.cell1"+r+c).className="";
                    }
                    document.getElementById("monthly.cell1"+r+c).onclick = Game.Screen.monthlyScreen._extendFunctionWithRender(Game.Screen.monthlyScreen._setupTopic(topic));
                    c++;
                    if (c===4) { r++; c=1; }
                }
            }
            
            r=1; c=1; letter='J';
            var subjects = Firstline.getViews(Game.Screen.monthlyScreen._reason.getCode(),Game.Screen.monthlyScreen._topic);
            for(i=0; i<subjects.length; i++) {
                var subject = subjects[i];
                if (Game.Screen.monthlyScreen._topic !== "Bribe" || Game._money >= Firstline.penceOf(subject)) {
                    Game.setIdText("monthly.cell2"+r+c,letter+". "+subject);
                    Game.Screen.monthlyScreen._choices[letter] = Game.Screen.monthlyScreen._setupSubject(subject,true);
                    letter = Game.nextChar(letter);
                    if (subject === Game.Screen.monthlyScreen._view) {
                        document.getElementById("monthly.cell2"+r+c).className="selectedoption";
                    } else {
                        document.getElementById("monthly.cell2"+r+c).className="";
                    }
                    document.getElementById("monthly.cell2"+r+c).onclick = Game.Screen.monthlyScreen._extendFunctionWithRender(Game.Screen.monthlyScreen._setupSubject(subject));
                    c++;
                    if (c===4) { r++; c=1; }
                }
            }
            
            r=1; c=1; letter='1';
            var attitudes = Firstline.getAttitudes(Game.Screen.monthlyScreen._reason.getCode(),Game.Screen.monthlyScreen._topic,Game.Screen.monthlyScreen._view);
            for(i=0; i<attitudes.length; i++) {
                var att = attitudes[i];
                Game.setIdText("monthly.cell3"+r+c,letter+". "+att);
                Game.Screen.monthlyScreen._choices[letter] = Game.Screen.monthlyScreen._setupAttitude(att,true);
                letter = Game.nextChar(letter);
                if (att === Game.Screen.monthlyScreen._attitude) {
                    document.getElementById("monthly.cell3"+r+c).className="selectedoption";
                } else {
                    document.getElementById("monthly.cell3"+r+c).className="";
                }
                document.getElementById("monthly.cell3"+r+c).onclick = Game.Screen.monthlyScreen._extendFunctionWithRender(Game.Screen.monthlyScreen._setupAttitude(att));
                c++;
                if (c===4) { r++; c=1; }
            }
            
            Game.clearNode("monthly.conversation");
            
            if (Game.Screen.monthlyScreen._conversation.length === 0) {
                Game.setIdText("monthly.next",
                        "Select a letter to choose a topic, subject or attitude or Press [Enter] to see the conversation.",
                        undefined,
                        function() { 
                            Game.captureException(Game.Screen.monthlyScreen._converse); 
                            Game.captureException(Game.Screen.monthlyScreen.render); 
                            Game.captureException(Game.renderExtra);
                        });
            } else {
                Game.Screen.monthlyScreen._conversation.forEach(function(v){
                    Game.addElementToNode("dt","monthly.conversation",undefined,v.actor,(v.actor === "YOU" ? "gebold" : "genonbold"));
                    Game.addElementToNode("dd","monthly.conversation",undefined,v.say,(v.actor === "YOU" ? "gebold" : "genonbold"));
                });
                // Implement the plan now, so the Chronicle is updated (and hence we can check if it's next)
                if (!Game.Screen.monthlyScreen._implemented) {
                    Game.Screen.monthlyScreen._changedVote = 
                        Game.implementPlan(Game.Screen.monthlyScreen._reason,
                            Game.Screen.monthlyScreen._topic,
                            Game.Screen.monthlyScreen._view,
                            Game.Screen.monthlyScreen._attitude,
                            Game.Screen.monthlyScreen._person,
                            Game.Screen.monthlyScreen._score);
                    Game.Screen.monthlyScreen._implemented = true;
                    var success = Game.Screen.monthlyScreen._changedVote;
                    var odds = (Game.Screen.monthlyScreen._score <=50 ? Game.Screen.monthlyScreen._score/2 : (100-Game.Screen.monthlyScreen._score)/2);
                    if (ROT.RNG.getPercentage() <= odds) {
                        success = !success; // Sometimes you're wrong - particularly if the score wasn't extreme :)
                    }
                    Game.Screen.monthlyScreen._result = (success ? "You think that %p will change their vote. " : "You don't think you've influenced %p's opinion enough to persuade them to change their vote.");
                    if (success) {
                        Game._estimatedVote++;
                    }
                }
                if (Game._month < 12) {
                    if (Game._news.length > 0) {
                        Game.setIdText("monthly.next",
                                "Press [Enter] to view the latest issue of the Chronicle.", 
                                undefined,
                                Game.Screen.monthlyScreen._nextMonth);
                    } else {
                        Address.displayData(Address.random())();
                        Address.activateMap(true);
                        Address.selectedChangeListener = function(address) {
                            Game.setIdText("monthly.next",
                                    "Press [Enter] to visit "+address.getAddress()+"  or double-click the address you wish to visit.", 
                                    undefined,
                                    Game.Screen.monthlyScreen._nextMonth);
                        };
                        Address.selectedChangeListener(Address.getSelected());
                        Address.selectedClickListener = function() {
                            Game.Screen.monthlyScreen._nextMonth();
                        };
                    }
                } else {
                    Game.setIdText("monthly.next",
                            "Press [Enter] to view the election results.",
                            undefined,
                            Game.Screen.monthlyScreen._nextMonth);
                }
            }
            var mnres = document.getElementById("monthly.result");
            if (Game.Screen.monthlyScreen._result) {
                mnres.className = mnres.className.replace(/mnhidden/,"mnvis");
                Game.setIdText("monthly.result.text",Game.Screen.monthlyScreen._result.format(Game.Screen.monthlyScreen._longReasonFormat,Game.Screen.monthlyScreen._longReasonFormat,Game.Screen.monthlyScreen._longReasonFormat));
            } else {
                mnres.className = mnres.className.replace(/mnvis/,"mnhidden");
            }
        },
        _converse: function() {
            var reason = Game.Screen.monthlyScreen._reason;
            var topic = Game.Screen.monthlyScreen._topic;
            var view = Game.Screen.monthlyScreen._view;
            var attitude = Game.Screen.monthlyScreen._attitude;
            var line = Firstline.getLine(reason,topic,view,attitude);
            var actor = Game.Screen.monthlyScreen._person;
            Game.Screen.monthlyScreen._score = actor.getScore(Game._factors,line);
            Game.Screen.monthlyScreen._conversation.push(
                    {actor: "YOU",
                    say:line.getText().format(Game.Screen.monthlyScreen._longReasonFormat,Game.Screen.monthlyScreen._longReasonFormat,Game.Screen.monthlyScreen._longReasonFormat)
                    });
            Game.Screen.monthlyScreen._conversation.push({actor: actor.getName(), say: actor.reply(Game._factors,Game.Screen.monthlyScreen._score,line).format(Game.Screen.monthlyScreen._longReasonFormat,Game.Screen.monthlyScreen._longReasonFormat,Game.Screen.monthlyScreen._longReasonFormat)});
        },
        _nextMonth: function() {
            Game.nextMonth();
            if (Game._month === 0) {
                Game.switchScreen(Game.Screen.electionScreen);
            } else if (Game._news.length > 0) {
                Game.switchScreen(Game.Screen.chronicleScreen);
            } else {
                Game.switchScreen(Game.Screen.monthlyScreen); // We want exit and enter to rerun
            }
        },
        handleInput: function(inputType, inputData) {
            if (inputType === 'keydown') {
                // If enter is pressed move onto planning
                if (inputData.keyCode === ROT.VK_RETURN) {
                    if (Game.Screen.monthlyScreen._conversation.length === 0) {
                        Game.Screen.monthlyScreen._converse();
                    } else {
                        Game.Screen.monthlyScreen._nextMonth();
                    }
                }
            } else if (inputType === 'keypress') {
                var code = inputData.charCode;
                var ch = String.fromCharCode(code).toUpperCase();
                if (Game.Screen.monthlyScreen._choices[ch]) {
                    Game.Screen.monthlyScreen._choices[ch]();
                }
            }
        }

};

// Define our winning screen
Game.Screen.winScreen = {
    name: "screen.win",
    enter: function() { },
    exit: function() { },
    render: function() {
        Game.setIdText("win.years",Game.Screen.electionScreen.plural(Game._year-136,"Year"));
    },
    handleInput: function(inputType, inputData) {
        // Nothing to do here      
    }
};
